---------------------------------------------------------------------------------------------------------------------
---------------------------------   SLATE CONNECTOR V 0.59 ALPHA   -----------------------------------------
---------------------------------------------------------------------------------------------------------------------
-- Author: David Almeida
-- Date: 01/13/2015
-- Version 0.59 alpha
-- Contributors (Alphabetical Order): Artur Leao, Clovis Gay, Martin Geupel, Paul Neale, Stig Atle Steffensen
-- Script Suggested by Marcelo Souza
---------------------------------------------------------------------------------------------------------------------
--------------------------------------------  CHANGELOG  ------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------
-- V 0.1a - First build
-- V 0.2a - Corrected some missing code for connecting and displaying maps
-- V 0.3a - Some UI changes
-- V 0.4a - Added the ability to change the materials selection without the need to open/close the script
-- V 0.51a - Major script overhaul, code cleaning and functions added to make it more "future proof"
-- V 0.53a - First autonaming tentative (Auto renaming created nodes)
-- V 0.54a - Vray supported and made the base-code for other render engines to be supported as well.
-- V 0.55a - Mray and Scanline partially supported (Arch&Design and Standard materials)
-- V 0.56a - Separated the script by render types, to better use each render peculiarities and material parameters
-- V 0.58a - Adjusted/fixed the unique/instancing method of the Node Creation group
-- V 0.59a - All 4 renderers supported and several bugs fixed. Still to find a way to auto-expand the AdditionalParams
--			  of materials without the need of user intervention.
---------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------
if (sme.GetView (sme.activeView)) != undefined then
(
	
	(local 
		CurRend = renderers.current as string,
		MatType = undefined,
		CurView = (sme.GetView (sme.activeView)),
		--SelMat = CurView.getSelectedNodes(),
		--SelMatCount = SelMat.count,
		SelNode = Color_RGB(),
		MatList = #(),
		MatListNames = #(),
		MatsString = "",
		token = 0,
		SlctParam = #colorDiffuse,
		NodeGrp = "RGB",
		SlctNode = Color_RGB(),
		-- Collects ALL scene materials who are either Standard, Vray or Corona
		--ScnMat = ((getClassInstances vraymtl) + (getClassInstances standardmaterial) + (getClassInstances coronamtl)),
		DisplayParams = #("No material selected"),
		SubDisplayParams = #(),
		MtlParams = #(),
		FullMtlParams =#(),
		SubMtlParams = #(),
		-- List of all CoronaMtl parameters exposed to be displayed by this script
		CrnDisplayParams = #("DiffuseColor(RGB)","DiffuseLevel(Float)","Diffuse(Map)","ReflectionColor(RGB)",\
			"ReflectionLevel(Float)","Reflection(Map)","ReflectionGlossiness(Float)","ReflectionGlossiness(Map)",\
			"IOR Fresnel(Map)","IOR Fresnel(Float)","RefractionColor(RGB)","RefractLevel(Float)","Refraction(Map)",\
			"RefractionGlossiness(Float)","RefractionGlossiness(Map)","IOR(Map)","IOR(Float)","Anisotropy(Map)",\
			"AnisotropyRotation(Map)","OpacityColor(RGB)","OpacityLevel(Float)","Opacity(Map)",\
			"TranslucencyColor(RGB)","TranslucencyLevel(Float)","Translucency(Map)","SelfIlluminationColor(RGB)",\
			"SelfIlluminationMultiplier(Float)","SelfIllumination(Map)","Bump(Map)","BumpAmount(Float)","Displacement(Map)"),
			
		-- List of all CoronaMtl RGB-type parameters exposed to be displayed by this script
		CrnDispRGBParams = #("DiffuseColor(RGB)","ReflectionColor(RGB)","RefractionColor(RGB)","OpacityColor(RGB)",\
			"TranslucencyColor(RGB)","SelfIlluminationColor(RGB)"),
			
		-- List of all CoronaMtl Float-type parameters exposed to be displayed by this script
		CrnDispFloatParams = #("DiffuseLevel(Float)","ReflectionLevel(Float)","ReflectionGlossiness(Float)",\
			"IOR Fresnel(Float)","RefractLevel(Float)","RefractionGlossiness(Float)","IOR(Float)","OpacityLevel(Float)",\
			"TranslucencyLevel(Float)","SelfIlluminationMultiplier(Float)","BumpAmount(Float)"),
			
		-- List of all CoronaMtl Map-type parameters exposed to be displayed by this script	
		CrnDispMapParams = #("Diffuse(Map)","Reflection(Map)","ReflectionGlossiness(Map)","IOR Fresnel(Map)",\
			"Refraction(Map)","RefractionGlossiness(Map)","IOR(Map)","Anisotropy(Map)","AnisotropyRotation(Map)",\
			"Opacity(Map)","Translucency(Map)","SelfIllumination(Map)","Bump(Map)","Displacement(Map)"),
			
		-- List of all CoronaMtl parameters exposed by this script	
		CrnMtlParams = #(#colorDiffuse, #levelDiffuse, #texmapDiffuse, #colorReflect, #levelReflect, #texmapReflect,\
			#reflectGlossiness,#texmapReflectGlossiness, #texmapFresnelIor, #fresnelIor, #colorRefract, #levelRefract,\
			#texmapRefract,#refractGlossiness,#texmapRefractGlossiness, #texmapIor, #ior, #texmapReflectAnisotropy,\
			#texmapReflectAnisotropyRotation, #colorOpacity, #levelOpacity,#texmapOpacity, #colorTranslucency,\
			#levelTranslucency, #texmapTranslucency, #colorSelfIllum, #levelSelfIllum, #texmapSelfIllum,#texmapBump,\
			#mapamountBump, #texmapDisplace),
			
		-- List of all CoronaMtl RGB-type parameters exposed by this script
		CrnRGBParams = #(#colorDiffuse,#colorReflect,#colorRefract,#colorOpacity,#colorTranslucency,#colorSelfIllum),
			
		-- List of all CoronaMtl Float-type parameters exposed by this script
		CrnFloatParams = #(#levelDiffuse,#levelReflect,#reflectGlossiness,#fresnelIor,#levelRefract,\
			#refractGlossiness,#ior,#levelOpacity,#levelTranslucency,#levelSelfIllum,#mapamountBump),
			
		-- List of all CoronaMtl Map-type parameters exposed by this script
		CrnMapParams = #(#texmapDiffuse,#texmapReflect,#texmapReflectGlossiness,#texmapFresnelIor,\
			#texmapRefract,#texmapRefractGlossiness,#texmapIor,#texmapReflectAnisotropy,\
			#texmapReflectAnisotropyRotation,#texmapOpacity,#texmapTranslucency,#texmapSelfIllum,\
			#texmapBump,#texmapDisplace),
			
		-- List of all VRayMtl parameters exposed to be displayed by this script
		VryDisplayParams = #("DiffuseColor(RGB)","Diffuse(Map)","DiffuseAmount(Float)","Roughness(Float)",\
			"RoughnessMap(Map)","ReflectionColor(RGB)","Reflection(Map)","ReflectionAmount(Float)",\
			"ReflectionGlossiness(Float)","ReflectionGlossiness(Map)","Fresnel IOR(Map)","Fresnel IOR(Float)",\
			"RefractionColor(RGB)","Refraction(Map)","RefractionAmount(Float)","RefractionGlossiness(Float)",\
			"RefractionGlossiness(Map)","IOR(Float)","IOR(Map)","Anisotropy(Map)","AnisotropyRotation(Map)",\
			"Opacity(Map)","OpacityAmount(Float)","TranslucencyColor(RGB)","Translucency(Map)",\
			"SelfIlluminationColor(RGB)","SelfIlluminationMultiplier(Float)","SelfIllumination(Map)","Bump(Map)",\
			"BumpAmount(Float)","Displacement(Map)","DisplacementAmount(Float)"),
			
		-- List of all VRayMtl RGB-type parameters exposed to be displayed by this script
		VryDispRGBParams = #("DiffuseColor(RGB)","ReflectionColor(RGB)","RefractionColor(RGB)",\
			"TranslucencyColor(RGB)","SelfIlluminationColor(RGB)"),
			
		-- List of all VRayMtl Float-type parameters exposed to be displayed by this script
		VryDispFloatParams = #("DiffuseAmount(Float)","Roughness(Float)","ReflectionAmount(Float)",\
			"ReflectionGlossiness(Float)","Fresnel IOR(Float)","RefractionAmount(Float)",\
			"RefractionGlossiness(Float)","IOR(Float)","OpacityAmount(Float)","SelfIlluminationMultiplier(Float)",\
			"BumpAmount(Float)","DisplacementAmount(Float)"),
			
		-- List of all VRayMtl Map-type parameters exposed to be displayed by this script
		VryDispMapParams = #("Diffuse(Map)","RoughnessMap(Map)","Reflection(Map)","ReflectionGlossiness(Map)",\
			"Fresnel IOR(Map)","Refraction(Map)","RefractionGlossiness(Map)","IOR(Map)","Anisotropy(Map)",\
			"AnisotropyRotation(Map)","Opacity(Map)","Translucency(Map)","SelfIllumination(Map)","Bump(Map)",\
			"Displacement(Map)"),
			
		-- List of all VRayMtl parameters exposed by this script
		VryMtlParams = #(#diffuse,#texmap_diffuse,#texmap_diffuse_multiplier,#diffuse_roughness,\
			#texmap_roughness,#Reflection,#texmap_reflection,#texmap_reflection_multiplier,\
			#reflection_glossiness,#texmap_reflectionGlossiness,#texmap_reflectionIOR,\
			#reflection_ior,#Refraction,#texmap_refraction,#texmap_refraction_multiplier,\
			#refraction_glossiness,#texmap_refractionGlossiness,#refraction_ior,#texmap_refractionIOR,\
			#texmap_anisotropy,#texmap_anisotropy_rotation,#texmap_opacity,#texmap_opacity_multiplier,\
			#translucency_color,#texmap_translucent,#selfIllumination,#selfIllumination_multiplier,\
			#texmap_self_illumination,#texmap_bump,#texmap_bump_multiplier,#texmap_displacement,\
			#texmap_displacement_multiplier),
			
		-- List of all VRayMtl RGB-type parameters exposed by this script
		VryRGBParams = #(#diffuse,#Reflection,#Refraction,#translucency_color,#selfIllumination),
			
		-- List of all VRayMtl Float-type parameters exposed by this script
		VryFloatParams = #(#texmap_diffuse_multiplier,#diffuse_roughness,#texmap_reflection_multiplier,\
			#reflection_glossiness,#reflection_ior,#texmap_refraction_multiplier,#refraction_glossiness,\
			#refraction_ior,#texmap_opacity_multiplier,#selfIllumination_multiplier,#texmap_bump_multiplier,\
			#texmap_displacement_multiplier),
			
		-- List of all VRayMtl Map-type parameters exposed by this script
		VryMapParams = #(#texmap_diffuse,#texmap_roughness,#texmap_reflection,\
			#texmap_reflectionGlossiness,#texmap_reflectionIOR,#texmap_refraction,\
			#texmap_refractionGlossiness,#texmap_refractionIOR,#texmap_anisotropy,\
			#texmap_anisotropy_rotation,#texmap_opacity,#texmap_translucent,#texmap_self_illumination,\
			#texmap_bump,#texmap_displacement),
			
		-- List of all Arch&DesignMtl (MentalRay) parameters exposed to be displayed by this script
		MrDisplayParams = #("DiffuseColor(RGB)","DiffuseLevel(Float)","Diffuse(Map)","ReflectionColor(RGB)",\
			"ReflectionLevel(Float)","Reflection(Map)","ReflectionGlossiness(Float)","ReflectionGlossiness(Map)",\
			"RefractionColor(RGB)","RefractLevel(Float)","Refraction(Map)","RefractionGlossiness(Float)",\
			"RefractionGlossiness(Map)","IOR(Map)","IOR(Float)","Anisotropy(Map)","AnisotropyRotation(Map)",\
			"Opacity(Map)","TranslucencyColor(RGB)","TranslucencyLevel(Float)","Translucency(Map)",\
			"SelfIlluminationColor(RGB)","SelfIlluminationMultiplier(Float)","SelfIllumination(Map)","Bump(Map)",\
			"BumpAmount(Float)","Displacement(Map)","DisplacementAmount(Float)"),
			
		-- List of all Arch&DesignMtl (MentalRay) RGB-type parameters exposed to be displayed by this script
		MrDispRGBParams = #("DiffuseColor(RGB)","ReflectionColor(RGB)","RefractionColor(RGB)",\
			"TranslucencyColor(RGB)","SelfIlluminationColor(RGB)"),
			
		-- List of all Arch&DesignMtl (MentalRay) Float-type parameters exposed to be displayed by this script
		MrDispFloatParams = #("DiffuseLevel(Float)","ReflectionLevel(Float)","ReflectionGlossiness(Float)",\
			"RefractLevel(Float)","RefractionGlossiness(Float)","IOR(Float)","TranslucencyLevel(Float)",\
			"SelfIlluminationMultiplier(Float)","BumpAmount(Float)","DisplacementAmount(Float)"),
			
		-- List of all Arch&DesignMtl (MentalRay) Map-type parameters exposed to be displayed by this script
		MrDispMapParams = #("Diffuse(Map)","Reflection(Map)","ReflectionGlossiness(Map)",\
			"Refraction(Map)","RefractionGlossiness(Map)","IOR(Map)","Anisotropy(Map)",\
			"AnisotropyRotation(Map)","Opacity(Map)","Translucency(Map)","SelfIllumination(Map)","Bump(Map)",\
			"Displacement(Map)"),
			
		-- List of all Arch&DesignMtl (MentalRay) parameters exposed by this script
		MrMtlParams = #(#diff_color,#diff_weight,#mapM0,#refl_color,#refl_weight,#mapM2,#refl_gloss,\
			#mapM3,#refr_color,#refr_weight,#mapM4,#refr_gloss,#mapM5,#mapM6,#refr_ior,#mapM9,\
			#mapM10,#cutout_map,#refr_transc,#refr_transw,#mapM7,#self_illum_color_filter,\
			#self_illum_int_arbitrary,#mapM23,#bump_map,#bump_map_amt,#displacement_map,\
			#displacement_map_amt),
			
		-- List of all Arch&DesignMtl (MentalRay) RGB-type parameters exposed by this script
		MrRGBParams = #(#diff_color,#refl_color,#refr_color,#refr_transc,#self_illum_color_filter),
			
		-- List of all Arch&DesignMtl (MentalRay) Float-type parameters exposed by this script
		MrFloatParams = #(#diff_weight,#refl_weight,#refl_gloss,#refr_weight,#refr_gloss,#refr_ior,\
			#refr_transw,#self_illum_int_arbitrary,#bump_map_amt,#displacement_map_amt),
			
		-- List of all Arch&DesignMtl (MentalRay) Map-type parameters exposed by this script
		MrMapParams =  #(#mapM0,#mapM2,#mapM3,#mapM4,#mapM5,#mapM6,#mapM9,#mapM10,\
			#cutout_map,#mapM7,#mapM23,#bump_map,#displacement_map),
			
		-- List of all StandardMtl parameters exposed to be displayed by this script
		StdDisplayParams = #("DiffuseColor(RGB)","Diffuse(Map)","SpecularLevel(Float)","Glossiness(Float)",\
			"Glossiness(Map)","Reflection(Map)","Refraction(Map)","OpacityLevel(Float)","Opacity(Map)",\
			"SelfIlluminationColor(RGB)","SelfIlluminationMultiplier(Float)","SelfIllumination(Map)","Bump(Map)",\
			"Displacement(Map)"),
			
		-- List of all StandardMtl RGB-type parameters exposed to be displayed by this script
		StdDispRGBParams = #("DiffuseColor(RGB)","SelfIlluminationColor(RGB)"),
			
		-- List of all StandardMtl Float-type parameters exposed to be displayed by this script
		StdDispFloatParams = #("SpecularLevel(Float)","Glossiness(Float)","OpacityLevel(Float)",\
			"SelfIlluminationMultiplier(Float)"),
			
		-- List of all StandardMtl Map-type parameters exposed to be displayed by this script
		StdDispMapParams = #("Diffuse(Map)","Glossiness(Map)","Reflection(Map)","Refraction(Map)",\
			"Opacity(Map)","SelfIllumination(Map)","Bump(Map)","Displacement(Map)"),
			
		-- List of all StandardMtl parameters exposed by this script
		StdMtlParams = #(#diffuse_color,#DiffuseMap,#specularLevel,#glossiness,#GlossinessMap,\
			#ReflectionMap,#RefractionMap,#opacity,#OpacityMap,#selfIllumColor,#selfIllumAmount,\
			#selfIllumMap,#bumpMap,#displacementMap),
			
		-- List of all StandardMtl RGB-type parameters exposed by this script
		StdRGBParams = #(#diffuse_color,#selfIllumColor),
			
		-- List of all StandardMtl Float-type parameters exposed by this script
		StdFloatParams = #(#specularLevel,#glossiness,#opacity,#selfIllumAmount),
			
		-- List of all StandardMtl Map-type parameters exposed by this script
		StdMapParams = #(#DiffuseMap,#GlossinessMap,#ReflectionMap,#RefractionMap,#OpacityMap,\
			#selfIllumMap,#bumpMap,#displacementMap),
			
		-- Node items Variable
		NodeItems = #(),
		
		-- List of supported Float Nodes to be displayed
		FloatItems = #("Bezier Float","Linear Float","Float Limit","Float Expression","Float Script"),
			
		-- List of supported Float Nodes
		FloatNodes = #(Bezier_Float(),Linear_Float(),Float_Limit(),Float_Expression(),Float_Script()),
			
		-- List of supported RGB (P3) Nodes to be displayed
		P3Items = #("Color RGB","Bezier Pt3","Pt3 Expression","Pt3 Script"),
			
		-- List of supported RGB (P3) Nodes
		P3Nodes = #(Color_RGB(),Bezier_Point3(),Point3_Expression(),Point3_Script()),
			
		-- List of supported RGBA (P4) Nodes to be displayed (MentalRay Only)
		P4Items = #("Color RGBA","Bezier Pt4","Pt4 Script"),
			
		-- List of supported RGBA (P4) Nodes (MentalRay Only)
		P4Nodes =#(Color_RGBA(),Bezier_Point4(),Point4_Script()),
			
		-- List of supported Map Nodes to be displayed
		MapItems = #("Bitmap","Camera Map","Cellular","Checker","Color Correction","Combustion","Composite",\
			"Dent","Falloff","Gradient","Gradient Ramp","Marble","Mask","Mix","Noise","Normal Bump","Output",\
			"Paticle Age","Particle MBlur","Perlin Marble","RGB Multiply","RGB Tint","Smoke","Speckle","Splat",\
			"Stucco","Substance","Swirl","Tiles","Vertex Color"),
			
		-- List of supported Map Nodes
		MapNodes = #(BitmapTex(),CameraMapTexture(),Cellular(),Checker(),ColorCorrection(),Combustion(),\
			CompositeMap(),Dent(),Falloff(),Gradient(),Gradient_Ramp(),Marble(),Mask(),Mix(),Noise(),\
			Normal_Bump(),Output(),Particle_Age(), Particle_Mblur(),PerlinMarble(),RGB_Multiply(),RGB_Tint(),\
			Smoke(),Speckle(),Splat(),Stucco(), Substance(), Swirl(), Tiles(), VertexColor()),
			
		TexMapON = #(),
			
		CrnTexMapON = #(#undefined,#undefined,#texmapOnDiffuse,#undefined,#undefined,#texmapOnReflect,\
			#undefined,#texmapOnReflectGlossiness,#texmapOnFresnelIor,#undefined,#undefined,#undefined,\
			#texmapOnRefract,#undefined,#texmapOnRefractGlossiness,#texmapOnIor,#undefined,\
			#texmapOnReflectAnisotropy,#texmapOnReflectAnisotropyRotation,#undefined,#undefined,\
			#texmapOnOpacity,#undefined,#undefined,#texmapOnTranslucency,#undefined,#undefined,\
			#texmapOnSelfIllum,#texmapOnBump,#undefined,#texmapOnDisplacement),
			
		SubCrnTexMapON = #(#texmapOnDiffuse,#texmapOnReflect,#texmapOnReflectGlossiness,\
			#texmapOnFresnelIor,#texmapOnRefract,#texmapOnRefractGlossiness,#texmapOnIor,\
			#texmapOnReflectAnisotropy,#texmapOnReflectAnisotropyRotation,#texmapOnOpacity,\
			#texmapOnTranslucency,#texmapOnSelfIllum,#texmapOnBump,#texmapOnDisplacement )
			
		MatsString = (MatList.count as string + " Material(s) Selected")
		try(destroyDialog SlateConnector) catch()
		rollout SlateConnector "Slate Connector V0.59A"
		(
			multilistbox theMats "Selected Materials:" items:MatListNames width:180 height:6
			label theLabel MatsString
			button theUpdater "Updt Selection" tooltip:"Updates the Material Selection"
			--List the current available parameters to select
			listbox theParams "Parameter:" items:DisplayParams width:180
			--List the current available nodes to select depending on the parameter selected
			listbox theNodes "NodeType:" items:NodeItems width:180
			group "Node Creation" (
			radiobuttons theUnique labels:#("Unique","Instanced")
			button MakeNodes "Make Nodes" --across:2
			--button DeleteNode "Delete Node"
			)
			on theUnique changed state do
			(
				if state == 1 then
				(
					theNodes.enabled = true
					theParams.items = DisplayParams
					MtlParams = FullMtlParams
					TexMapON = CrnTexMapON
				)
				else
				(
					theNodes.enabled = false
					theParams.items = SubDisplayParams
					MtlParams = SubMtlParams
					TexMapON = SubCrnTexMapON
				)
			)
			on theUpdater pressed do
			(
				CurView = (sme.GetView (sme.activeView))
				SelMat = CurView.getSelectedNodes()
				MatList = #()
				MatListNames = #()
				theUnique.state = 1
				theNodes.enabled = true
				--DisplayParams = #("No material selected")
				--NodeItems = #()
				CurRend = renderers.current as string
				-- Checks which the current renderer is (Scanline, Vray/VrayRT, Corona or MentalRay/Iray)
				case of
				(
					(matchpattern CurRend pattern:"*Scanline*"):
					(
						MatType = Standardmaterial
						MtlParams = FullMtlParams = StdMtlParams
						SubMtlParams = StdRGBParams
						DisplayParams = StdDisplayParams
						SubDisplayParams = StdDispRGBParams
						NodeItems = P3Items
					)
					(matchpattern CurRend pattern:"V_Ray*"):
					(
						MatType = VRayMtl
						MtlParams = FullMtlParams = VryMtlParams
						SubMtlParams = VryRGBParams
						DisplayParams = VryDisplayParams
						SubDisplayParams = VryDispRGBParams
						NodeItems = P3Items
					)
					(matchpattern CurRend pattern:"Corona*"):
					(
						MatType = CoronaMtl
						MtlParams = FullMtlParams = CrnMtlParams
						SubMtlParams = CrnRGBParams
						DisplayParams = CrnDisplayParams
						SubDisplayParams = CrnDispRGBParams
						NodeItems = P3Items
						TexMapON = CrnTexMapON
					)
					((matchpattern CurRend pattern:"iray*") or (matchpattern CurRend pattern:"mental_ray*")):
					(
						MatType = Arch___Design__mi
						MtlParams = FullMtlParams = MrMtlParams
						SubMtlParams = MrRGBParams
						DisplayParams = MrDisplayParams
						SubDisplayParams = MrDispRGBParams
						NodeItems = P4Items
					)
				)
				-- Loops through all the selected materials on Slate
				for SelMatName in SelMat do
				(
					-- Loops through all the material nodes on the TrackView and gets the SuperClass and Class of the nodes
					for i = 1 to trackViewNodes[#sme][1].numsubs do
					(
						NodeClass = (SuperClassOf (trackViewNodes[#sme][1][i].reference))
						NodeType = (ClassOf (trackViewNodes[#sme][1][i].reference))
						-- Checks if the Superclass of the looped nodes are of the "material" type, if their Class 
						-- corresponds to the current render Material Class and if the names of the Selected Materials 
						-- on the Slate are the same of the ones on the TrackView
						if (NodeClass == material) and (NodeType == MatType) and (SelMatName.name == trackViewNodes[#sme][1][i].reference.name) then
						(
							append MatList trackViewNodes[#sme][1][i].reference
							append MatListNames trackViewNodes[#sme][1][i].reference.name
						)
					)
				)
				-- Checks if a Material has been added to the material list and updates the "Parameter" and "NodeType" ListBoxes
				if MatList.count != 0 then
				(
					theParams.items = DisplayParams
					theNodes.items = NodeItems
				)
				else
				(
					MatType = undefined
					theParams.items = #("No material selected")
					theNodes.items = #()
				)
				-- Updates the text under the "Selected Materials" Multilistbox
				theMats.items = MatListNames
				theLabel.text = (MatList.count as string + " Material(s) Selected")
			)
			on theParams selected Param do
			(
				case of
				(
					(matchpattern theParams.selected pattern:"*(RGB)"):
					(
						theNodes.items = P3Items
						NodeGrp = "RGB"
						case of
						(
							(MatType == Standardmaterial):
							(
								SubMtlParams = StdRGBParams
								SubDisplayParams = StdDispRGBParams
							)
							(MatType == VRayMtl):
							(
								SubMtlParams = VryRGBParams
								SubDisplayParams = VryDispRGBParams
							)
							(MatType == CoronaMtl):
							(
								SubMtlParams = CrnRGBParams
								SubDisplayParams = CrnDispRGBParams
							)
							(MatType == Arch___Design__mi):
							(
								SubMtlParams = MrRGBParams
								SubDisplayParams = MrDispRGBParams
							)
						)
					)
					(matchpattern theParams.selected pattern:"*(Float)"):
					(
						theNodes.items = FloatItems
						NodeGrp = "Float"
						case of
						(
							(MatType == Standardmaterial):
							(
								SubMtlParams = StdFloatParams
								SubDisplayParams = StdDispFloatParams
							)
							(MatType == VRayMtl):
							(
								SubMtlParams = VryFloatParams
								SubDisplayParams = VryDispFloatParams
							)
							(MatType == CoronaMtl):
							(
								SubMtlParams = CrnFloatParams
								SubDisplayParams = CrnDispFloatParams
							)
							(MatType == Arch___Design__mi):
							(
								SubMtlParams = MrFloatParams
								SubDisplayParams = MrDispFloatParams
							)
						)
					)
					(matchpattern theParams.selected pattern:"*(Map)"):
					(
						theNodes.items = MapItems
						NodeGrp = "Map"
						case of
						(
							(MatType == Standardmaterial):
							(
								SubMtlParams = StdMapParams
								SubDisplayParams = StdDispMapParams
							)
							(MatType == VRayMtl):
							(
								SubMtlParams = VryMapParams
								SubDisplayParams = VryDispMapParams
							)
							(MatType == CoronaMtl):
							(
								SubMtlParams = CrnMapParams
								SubDisplayParams = CrnDispMapParams
							)
							(MatType == Arch___Design__mi):
							(
								SubMtlParams = MrMapParams
								SubDisplayParams = MrDispMapParams
							)
						)
					)
				)
			)
			on MakeNodes pressed do
			(
				if MatList.count == 0 then
				(
					messagebox "Select one or more materials on Slate\n and press the \"Updt Selection\" button" title:"Error"
				)
				else
				(
					if theNodes.selection == 0 then
					(
						messagebox "Select a Node Type" title:"Error"
					)
					else
					(
						OldNodeIndex = CurView.GetNumNodes()
						case of 
						(
							(NodeGrp == "Map"):
							(
								if theUnique.state == 1 then
								(
									MapNodes = #(BitmapTex(),CameraMapTexture(),Cellular(),Checker(),ColorCorrection(),Combustion(),CompositeMap(),Dent(),\
									Falloff(),Gradient(),Gradient_Ramp(),Marble(),Mask(),Mix(),Noise(),Normal_Bump(),Output(),Particle_Age(), Particle_Mblur(),\
									PerlinMarble(),RGB_Multiply(),RGB_Tint(),Smoke(),Speckle(),Splat(),Stucco(), Substance(), Swirl(), Tiles(), VertexColor())
								)
								if MatType == CoronaMtl then
								(
									for i = 1 to Matlist.count do
									(
										SetProperty MatList[i] MtlParams[theParams.selection] (MapNodes[theNodes.selection])
										SetProperty MatList[i] TexMapON[theParams.selection] true
									)
								)
								else
								(
									for i = 1 to Matlist.count do
									(
										SetProperty MatList[i] MtlParams[theParams.selection] (MapNodes[theNodes.selection])
									)
								)
								NewNodeIndex = CurView.GetNumNodes()
								if NewNodeIndex != OldNodeIndex then
								(
									(CurView.GetNode NewNodeIndex).name = theParams.selected
								)
							)
							(NodeGrp == "RGB"):
							(
								if theUnique.state == 1 then
								(
									P3Nodes = #(Color_RGB(),Bezier_Point3(),Point3_Expression(),Point3_Script())
									P4Nodes = #(Color_RGBA(), Bezier_Point4(),Point4_Script())
								)
								if MatType == Arch___Design__mi then
								(
									for i = 1 to Matlist.count do
									(
										SetPropertyController MatList[i] MtlParams[theParams.selection] (P4Nodes[theNodes.selection])
									)
								)
								else
								(
									for i = 1 to Matlist.count do
									(
										SetPropertyController MatList[i] MtlParams[theParams.selection] (P3Nodes[theNodes.selection])
									)
								)
								NewNodeIndex = CurView.GetNumNodes()
								if NewNodeIndex != OldNodeIndex then
								(
									(CurView.GetNode NewNodeIndex).name = theParams.selected
								)
								else
								(
									if theUnique.state == 1 then
									(
										messagebox "Select your materials, right-click,\nand select 'Show All Additional Params'"
									)
								)
							)
							(NodeGrp == "Float"):
							(
								if theUnique.state == 1 then
								(
									FloatNodes = #(Bezier_Float(),Linear_Float(),Float_Limit(),Float_Expression(),Float_Script())
								)
								for i = 1 to Matlist.count do
								(
									SetPropertyController MatList[i] MtlParams[theParams.selection] (FloatNodes[theNodes.selection])
									print "Worked"
								)
								NewNodeIndex = CurView.GetNumNodes()
								if NewNodeIndex != OldNodeIndex then
								(
									(CurView.GetNode NewNodeIndex).name = theParams.selected
								)
								else
								(
									if theUnique.state == 1 then
									(
										messagebox "Select your materials, right-click,\nand select 'Show All Additional Params'"
									)
								)
							)
						)
					)
				)
			)
			/*
			================================================================================================================================
			===================================================Future DELETE NODE Button================================================
			================================================================================================================================
			on DeleteNode pressed do
			(
				case of 
				(
					(NodeGrp == "Map"):
					(
						if MatType == CoronaMtl then
						(
							for i = 1 to Matlist.count do
							(
								SetProperty MatList[i] MtlParams[theParams.selection] (undefined)
								SetProperty MatList[i] CrnTexMapON[theParams.selection] false
							)
						)
						else
						(
							SetProperty MatList[i] MtlParams[theParams.selection] (undefined)
						)
					)
					(NodeGrp == "RGB"):
					(
						for i = 1 to Matlist.count do
						(
							SetPropertyController MatList[i] MtlParams[theParams.selection] (undefined)
						)
					)
					(NodeGrp == "Float"):
					(
						for i = 1 to Matlist.count do
						(
							SetPropertyController MatList[i] MtlParams[theParams.selection] (undefined)
						)
					)
				)
			)
			*/
		)
		createDialog SlateConnector width:210
	)
)
else
(
	messagebox "Open Slate Material Editor" title:"Very funny ;)"
)